@@ -13,10 +13,11 @@ from sqlalchemy import Table
 from sqlalchemy import UniqueConstraint
 
 from alembic.testing import assertions
+from alembic.testing import combinations
 from alembic.testing import config
-from alembic.testing import engines
 from alembic.testing import eq_
 from alembic.testing import TestBase
+from alembic.testing import util
 from alembic.testing.env import staging_env
 from alembic.util import sqla_compat
 from ._autogen_fixtures import AutogenFixtureTest
@@ -29,7 +30,7 @@ class NoUqReflection(object):
 
     def setUp(self):
         staging_env()
-        self.bind = eng = engines.testing_engine()
+        self.bind = eng = util.testing_engine()
 
         def unimpl(*arg, **kw):
             raise NotImplementedError()
@@ -365,6 +366,122 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestBase):
         diffs = self._fixture(m1, m2, max_identifier_length=30)
         eq_(diffs, [])
 
+    def test_nothing_changed_uq_w_mixed_case_nconv_name(self):
+        m1 = MetaData(
+            naming_convention={
+                "ix": "index_%(table_name)s_%(column_0_label)s",
+                "uq": "unique_%(table_name)s_%(column_0_label)s",
+            }
+        )
+        m2 = MetaData(
+            naming_convention={
+                "ix": "index_%(table_name)s_%(column_0_label)s",
+                "uq": "unique_%(table_name)s_%(column_0_label)s",
+            }
+        )
+
+        Table(
+            "NothingChanged",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("XCol", Integer),
+            UniqueConstraint("XCol"),
+            mysql_engine="InnoDB",
+        )
+
+        Table(
+            "NothingChanged",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("XCol", Integer),
+            UniqueConstraint("XCol"),
+            mysql_engine="InnoDB",
+        )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_nothing_changed_uq_w_mixed_case_plain_name(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            "nothing_changed",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            UniqueConstraint("x", name="SomeConstraint"),
+            mysql_engine="InnoDB",
+        )
+
+        Table(
+            "nothing_changed",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            UniqueConstraint("x", name="SomeConstraint"),
+            mysql_engine="InnoDB",
+        )
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_nothing_changed_ix_w_mixed_case_plain_name(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            "nothing_changed",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            Index("SomeIndex", "x"),
+            mysql_engine="InnoDB",
+        )
+
+        Table(
+            "nothing_changed",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            Index("SomeIndex", "x"),
+            mysql_engine="InnoDB",
+        )
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_nothing_changed_ix_w_mixed_case_nconv_name(self):
+        m1 = MetaData(
+            naming_convention={
+                "ix": "index_%(table_name)s_%(column_0_label)s",
+                "uq": "unique_%(table_name)s_%(column_0_label)s",
+            }
+        )
+        m2 = MetaData(
+            naming_convention={
+                "ix": "index_%(table_name)s_%(column_0_label)s",
+                "uq": "unique_%(table_name)s_%(column_0_label)s",
+            }
+        )
+
+        Table(
+            "NothingChanged",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("XCol", Integer, index=True),
+            mysql_engine="InnoDB",
+        )
+
+        Table(
+            "NothingChanged",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("XCol", Integer, index=True),
+            mysql_engine="InnoDB",
+        )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
     def test_nothing_changed_two(self):
         m1 = MetaData()
         m2 = MetaData()
@@ -1063,7 +1180,7 @@ class PGUniqueIndexTest(AutogenerateUniqueIndexTest):
 class MySQLUniqueIndexTest(AutogenerateUniqueIndexTest):
     reports_unnamed_constraints = True
     reports_unique_constraints_as_indexes = True
-    __only_on__ = "mysql"
+    __only_on__ = "mysql", "mariadb"
     __backend__ = True
 
     def test_removed_idx_index_named_as_column(self):
@@ -1192,7 +1309,8 @@ class NoUqReportsIndAsUqTest(NoUqReflectionIndexTest):
 class IncludeHooksTest(AutogenFixtureTest, TestBase):
     __backend__ = True
 
-    def test_remove_connection_index(self):
+    @combinations(("name",), ("object",))
+    def test_remove_connection_index(self, hook_type):
         m1 = MetaData()
         m2 = MetaData()
 
@@ -1202,25 +1320,59 @@ class IncludeHooksTest(AutogenFixtureTest, TestBase):
 
         Table("t", m2, Column("x", Integer), Column("y", Integer))
 
-        def include_object(object_, name, type_, reflected, compare_to):
-            if type_ == "unique_constraint":
-                return False
-            return not (
-                isinstance(object_, Index)
-                and type_ == "index"
-                and reflected
-                and name == "ix1"
-            )
+        if hook_type == "object":
 
-        diffs = self._fixture(m1, m2, object_filters=include_object)
+            def include_object(object_, name, type_, reflected, compare_to):
+                if type_ == "unique_constraint":
+                    return False
+                return not (
+                    isinstance(object_, Index)
+                    and type_ == "index"
+                    and reflected
+                    and name == "ix1"
+                )
+
+            diffs = self._fixture(m1, m2, object_filters=include_object)
+        elif hook_type == "name":
+            all_names = set()
+
+            def include_name(name, type_, parent_names):
+                all_names.add((name, type_))
+                if name == "ix1":
+                    eq_(type_, "index")
+                    eq_(
+                        parent_names,
+                        {
+                            "table_name": "t",
+                            "schema_name": None,
+                            "schema_qualified_table_name": "t",
+                        },
+                    )
+                    return False
+                else:
+                    return True
+
+            diffs = self._fixture(m1, m2, name_filters=include_name)
+            eq_(
+                all_names,
+                {
+                    ("ix1", "index"),
+                    ("ix2", "index"),
+                    ("y", "column"),
+                    ("t", "table"),
+                    (None, "schema"),
+                    ("x", "column"),
+                },
+            )
 
         eq_(diffs[0][0], "remove_index")
         eq_(diffs[0][1].name, "ix2")
         eq_(len(diffs), 1)
 
+    @combinations(("name",), ("object",))
     @config.requirements.unique_constraint_reflection
     @config.requirements.reflects_unique_constraints_unambiguously
-    def test_remove_connection_uq(self):
+    def test_remove_connection_uq(self, hook_type):
         m1 = MetaData()
         m2 = MetaData()
 
@@ -1235,17 +1387,53 @@ class IncludeHooksTest(AutogenFixtureTest, TestBase):
 
         Table("t", m2, Column("x", Integer), Column("y", Integer))
 
-        def include_object(object_, name, type_, reflected, compare_to):
-            if type_ == "index":
-                return False
-            return not (
-                isinstance(object_, UniqueConstraint)
-                and type_ == "unique_constraint"
-                and reflected
-                and name == "uq1"
-            )
+        if hook_type == "object":
 
-        diffs = self._fixture(m1, m2, object_filters=include_object)
+            def include_object(object_, name, type_, reflected, compare_to):
+                if type_ == "index":
+                    return False
+                return not (
+                    isinstance(object_, UniqueConstraint)
+                    and type_ == "unique_constraint"
+                    and reflected
+                    and name == "uq1"
+                )
+
+            diffs = self._fixture(m1, m2, object_filters=include_object)
+        elif hook_type == "name":
+            all_names = set()
+
+            def include_name(name, type_, parent_names):
+                if type_ == "index":
+                    return False  # PostgreSQL thing
+
+                all_names.add((name, type_))
+
+                if name == "uq1":
+                    eq_(type_, "unique_constraint")
+                    eq_(
+                        parent_names,
+                        {
+                            "table_name": "t",
+                            "schema_name": None,
+                            "schema_qualified_table_name": "t",
+                        },
+                    )
+                    return False
+                return True
+
+            diffs = self._fixture(m1, m2, name_filters=include_name)
+            eq_(
+                all_names,
+                {
+                    ("t", "table"),
+                    (None, "schema"),
+                    ("uq2", "unique_constraint"),
+                    ("x", "column"),
+                    ("y", "column"),
+                    ("uq1", "unique_constraint"),
+                },
+            )
 
         eq_(diffs[0][0], "remove_constraint")
         eq_(diffs[0][1].name, "uq2")
@@ -1392,7 +1580,7 @@ class IncludeHooksTest(AutogenFixtureTest, TestBase):
 
 class TruncatedIdxTest(AutogenFixtureTest, TestBase):
     def setUp(self):
-        self.bind = engines.testing_engine()
+        self.bind = util.testing_engine()
         self.bind.dialect.max_identifier_length = 30
 
     def test_idx_matches_long(self):
